@@ -44,6 +44,7 @@ static void ts_lua_inject_server_request_server_addr_api(lua_State *L);
 
 static void ts_lua_inject_server_request_socket_api(lua_State *L);
 static void ts_lua_inject_server_request_header_api(lua_State *L);
+static void ts_lua_inject_server_request_header_table_api(lua_State *L);
 static void ts_lua_inject_server_request_headers_api(lua_State *L);
 static void ts_lua_inject_server_request_get_header_size_api(lua_State *L);
 static void ts_lua_inject_server_request_get_body_size_api(lua_State *L);
@@ -55,6 +56,8 @@ static void ts_lua_inject_server_request_method_api(lua_State *L);
 
 static int ts_lua_server_request_header_get(lua_State *L);
 static int ts_lua_server_request_header_set(lua_State *L);
+static int ts_lua_server_request_header_table_get(lua_State *L);
+static int ts_lua_server_request_header_table_set(lua_State *L);
 static int ts_lua_server_request_get_headers(lua_State *L);
 static int ts_lua_server_request_get_header_size(lua_State *L);
 static int ts_lua_server_request_get_body_size(lua_State *L);
@@ -89,6 +92,7 @@ ts_lua_inject_server_request_api(lua_State *L)
 
   ts_lua_inject_server_request_socket_api(L);
   ts_lua_inject_server_request_header_api(L);
+  ts_lua_inject_server_request_header_table_api(L);
   ts_lua_inject_server_request_headers_api(L);
   ts_lua_inject_server_request_get_header_size_api(L);
   ts_lua_inject_server_request_get_body_size_api(L);
@@ -289,6 +293,82 @@ ts_lua_server_request_header_set(lua_State *L)
   return 0;
 }
 
+static void
+ts_lua_inject_server_request_header_table_api(lua_State *L)
+{
+  lua_newtable(L); /* .header */
+
+  lua_createtable(L, 0, 2); /* metatable for .header */
+
+  lua_pushcfunction(L, ts_lua_server_request_header_table_get);
+  lua_setfield(L, -2, "__index");
+  lua_pushcfunction(L, ts_lua_server_request_header_table_set);
+  lua_setfield(L, -2, "__newindex");
+
+  lua_setmetatable(L, -2);
+
+  lua_setfield(L, -2, "header_table");
+}
+
+static int
+ts_lua_server_request_header_table_get(lua_State *L)
+{
+  const char *key;
+  const char *val;
+  int val_len;
+  size_t key_len;
+  int count;
+
+  TSMLoc field_loc, next_field_loc;
+  ts_lua_http_ctx *http_ctx;
+
+  GET_HTTP_CONTEXT(http_ctx, L);
+
+  /*  we skip the first argument that is the table */
+  key = luaL_checklstring(L, 2, &key_len);
+
+  if (!http_ctx->server_request_hdrp) {
+    if (TSHttpTxnServerReqGet(http_ctx->txnp, &http_ctx->server_request_bufp, &http_ctx->server_request_hdrp) != TS_SUCCESS) {
+      lua_pushnil(L);
+      return 1;
+    }
+  }
+
+  if (key && key_len) {
+    field_loc = TSMimeHdrFieldFind(http_ctx->server_request_bufp, http_ctx->server_request_hdrp, key, key_len);
+
+    if (field_loc != TS_NULL_MLOC) {
+      lua_newtable(L);
+      count = 0;
+      while (field_loc != TS_NULL_MLOC) {
+        val = TSMimeHdrFieldValueStringGet(http_ctx->server_request_bufp, http_ctx->server_request_hdrp, field_loc, -1, &val_len);
+        next_field_loc = TSMimeHdrFieldNextDup(http_ctx->server_request_bufp, http_ctx->server_request_hdrp, field_loc);
+        count++;
+
+        lua_pushlstring(L, val, val_len);
+        lua_rawseti(L, -2, count);
+
+        TSHandleMLocRelease(http_ctx->server_request_bufp, http_ctx->server_request_hdrp, field_loc);
+        field_loc = next_field_loc;
+      }
+
+    } else {
+      lua_pushnil(L);
+    }
+
+  } else {
+    lua_pushnil(L);
+  }
+
+  return 1;
+}
+
+static int
+ts_lua_server_request_header_table_set(lua_State *L)
+{
+  return 0;
+}
+
 static void
 ts_lua_inject_server_request_headers_api(lua_State *L)
 {
